 % read data
A=xlsread('cigarette.xls');
W1=xlsread('Spat-Sym-US.xls'); % This is a binary contiguity matrix
% set dimensions
T=30; % number of time periods
N=46; % number of areas
% row-normalize W
% define dependent and independent variables
y=A(:,3); % column number in the data matrix that corresponds to the dependent variable
x=A(:,[4,6]); % column numbers in the data matrix that correspond to the independent variables
%
%
% PART I: Estimation of different static spatial panel data models with W=BC matrix
%
%
W=normw(W1); % row-normalize binary contiguity matrix
% Create WX variables
for t=1:T
    t1=(t-1)*N+1;t2=t*N;
    wx(t1:t2,:)=W*x(t1:t2,:);
end
% determine size of the x matrix
[nobs K]=size(x);
display('Results Binary Contiguity matrix')
%
% All models include spatial and time period fixed effects (sfe and tfe), (info.)model=3
% model=0 no sfe and tfe, model=1 only sfe, model=2 only tfe
% info.bc=1; % bias correction of Lee and Yu is turned on in all calculations
% ----------------------------------------------------------------------------------------
% OLS model without spatial interaction effects
model=3; % sfe + tfe
[ywith,xwith,meanny,meannx,meanty,meantx]=demean(y,x,N,T,model);
results=ols(ywith,xwith);
vnames=strvcat('logcit','logp','logy'); % should be changed if x is changed
prt_reg(results,vnames);
% Additional results
et=ones(T,1);
en=ones(N,1);
intercept=mean(y)-mean(x)*results.beta; 
sfe=meanny-meannx*results.beta-kron(en,intercept);
tfe=meanty-meantx*results.beta-kron(et,intercept);
yme = y - mean(y);
ent=ones(N*T,1);
error=y-kron(tfe,en)-kron(et,sfe)-x*results.beta-kron(ent,intercept);
rsqr1 = error'*error;
rsqr2 = yme'*yme;
FE_rsqr2 = 1.0 - rsqr1/rsqr2 % r-squared including fixed effects
sige=results.sige*((nobs-K)/nobs);
loglik=-nobs/2*log(2*pi*sige)-1/(2*sige)*results.resid'*results.resid
% Direct and indirect effects already determined by OLS
% ----------------------------------------------------------------------------------------
% Spatial lag model SAR
info.lflag=0; % required for exact results
info.model=3; % sfe + tfe
info.fe=0; % Do not print intercept and fixed effects; use info.fe=1 to turn on
results=sar_panel_FE(y,x,W,T,info); 
% Print out coefficient estimates
vnames=strvcat('logcit','logp','logy'); % variable names
prt_sp(results,vnames,1);
% Print out direct and indirect effects
spat_model=0; % 0 for SAR/SAC, 1 for SDM/GNS model
direct_indirect_effects_estimates(results,W,spat_model);
% ----------------------------------------------------------------------------------------
% Spatial error model SEM
info.lflag=0; % required for exact results
info.model=3; % sfe + tfe
info.fe=0; % Do not print intercept and fixed effects; use info.fe=1 to turn on
results=sem_panel_FE(y,x,W,T,info); 
% Print out coefficient estimates
vnames=strvcat('logcit','logp','logy'); % variable names
prt_sp(results,vnames,1);
% Direct and indirect effects already determined by SEM
% ----------------------------------------------------------------------------------------
% SLX Model
model=3; % sfe + tfe
[ywith,xwith,meanny,meannx,meanty,meantx]=demean(y,[x wx],N,T,model);
results=ols(ywith,xwith);
vnames=strvcat('logcit','logp','logy','W*logp','W*logy'); % variable names
prt_reg(results,vnames);
% Additional results
intercept=mean(y)-mean([x wx])*results.beta; 
sfe=meanny-meannx*results.beta-kron(en,intercept);
tfe=meanty-meantx*results.beta-kron(et,intercept);
yme = y - mean(y);
ent=ones(N*T,1);
error=y-kron(tfe,en)-kron(et,sfe)-[x wx]*results.beta-kron(ent,intercept);
rsqr1 = error'*error;
rsqr2 = yme'*yme;
FE_rsqr2 = 1.0 - rsqr1/rsqr2 % r-squared including fixed effects
sige=results.sige*((nobs-K)/nobs);
loglik=-nobs/2*log(2*pi*sige)-1/(2*sige)*results.resid'*results.resid
% Direct and indirect effects already determined by SLX
% ----------------------------------------------------------------------------------------
% Spatial Durbin model SDM
info.lflag=0; % required for exact results
info.model=3; % sfe + tfe
info.fe=0; % Do not print intercept and fixed effects; use info.fe=1 to turn on
results=sar_panel_FE(y,[x wx],W,T,info); 
% Print out coefficient estimates
vnames=strvcat('logcit','logp','logy','W*logp','W*logy'); % variable names
prt_sp(results,vnames,1);
% Print out direct and indirect effects
spat_model=1; % 0 for SAR/SAC, 1 for SDM/GNS model
direct_indirect_effects_estimates(results,W,spat_model);
% ----------------------------------------------------------------------------------------
% Spatial Durbin error model SDEM
info.lflag=0; % required for exact results
info.model=3; % sfe + tfe
info.fe=0; % Do not print intercept and fixed effects; use info.fe=1 to turn on
results=sem_panel_FE(y,[x wx],W,T,info); 
% Print out coefficient estimates
vnames=strvcat('logcit','logp','logy','W*logp','W*logy'); % variable names
prt_sp(results,vnames,1);
% Direct and indirect effects already determined by SDEM
% ----------------------------------------------------------------------------------------
% SARAR = SAC model
info.lflag=0; % required for exact results
info.model=3; % sfe + tfe
info.fe=0; % Do not print intercept and fixed effects; use info.fe=1 to turn on
results=sac_panel_FE(y,x,W,T,info);
% Print out coefficient estimates
vnames=strvcat('logcit','logp','logy'); % variable names
prt_sp(results,vnames,1);
% Print out direct and indirect effects
spat_model=0; % 0 for SAR/SAC, 1 for SDM/GNS model
direct_indirect_effects_estimates(results,W,spat_model);
% ----------------------------------------------------------------------------------------
% General nesting spatial model GNS
info.lflag=0; % required for exact results
info.model=3; % sfe + tfe
info.fe=0; % Do not print intercept and fixed effects; use info.fe=1 to turn on
results=sac_panel_FE(y,[x wx],W,T,info);
% Print out coefficient estimates
vnames=strvcat('logcit','logp','logy','W*logp','W*logy'); % variable names
prt_sp(results,vnames,1);
% Print out direct and indirect effects
spat_model=1; % 0 for SAR/SAC, 1 for SDM/GNS model
direct_indirect_effects_estimates(results,W,spat_model);
%
%
% END PART I: Estimation of different static spatial panel data models with W=BC matrix
% 
% Remove stop in line below after you answered questions 1-4.
%stop=firstpart

%
% PART II: Bayesian comparison tests, with W=BC, ID, ID^3
%
%
% Read in data to construct inverse distance matrix
%
display('Results Inverse Distance matrix')
B=xlsread('cigar_states.xls'); % reads in x and y coordinates
WID=zeros(N,N);
% computes inverse (Euclidian) distance matrix using theorem of Pythagoras
for i=1:N
    for j=1:N
        if (i==j) 
            WID(i,j)=0;
        else
            WID(i,j)=1/sqrt((B(i,1)-B(j,1))^2+(B(i,2)-B(j,2))^2); %Pythagoras
            D(i,j)=sqrt((B(i,1)-B(j,1))^2+(B(i,2)-B(j,2))^2);
        end
    end
end
% Normalizes W by largest eigenvalue
lambda=eig(WID);
WID=WID/max(lambda); 
%
% Create WX variables for WID matrix
for t=1:T
    t1=(t-1)*N+1;t2=t*N;
    wx(t1:t2,:)=WID*x(t1:t2,:);
end
%
% Estimation of SDM, SDEM, and GNS models using inverse distance matrix WID
%
% ----------------------------------------------------------------------------------------
% Spatial Durbin model SDM
info.lflag=0; % required for exact results
info.model=3; % sfe + tfe
info.fe=0; % Do not print intercept and fixed effects; use info.fe=1 to turn on
results=sar_panel_FE(y,[x wx],WID,T,info); 
% Print out coefficient estimates
vnames=strvcat('logcit','logp','logy','W*logp','W*logy'); % variable names
prt_sp(results,vnames,1);
% Print out direct and indirect effects
spat_model=1; % 0 for SAR/SAC, 1 for SDM/GNS model
direct_indirect_effects_estimates(results,WID,spat_model);
% ----------------------------------------------------------------------------------------
% Spatial Durbin error model
info.lflag=0; % required for exact results
info.model=3; % sfe + tfe
info.fe=0; % Do not print intercept and fixed effects; use info.fe=1 to turn on
results=sem_panel_FE(y,[x wx],WID,T,info); 
% Print out coefficient estimates
vnames=strvcat('logcit','logp','logy','W*logp','W*logy'); % variable names
prt_sp(results,vnames,1);
% Direct and indirect effects already determined by SDEM
% ----------------------------------------------------------------------------------------
% General nesting spatial model GNS
info.lflag=0; % required for exact results
info.model=3; % sfe + tfe
info.fe=0; % Do not print intercept and fixed effects; use info.fe=1 to turn on
results=sac_panel_FE(y,[x wx],WID,T,info);
% Print out coefficient estimates
vnames=strvcat('logcit','logp','logy','W*logp','W*logy'); % variable names
prt_sp(results,vnames,1);
% Print out direct and indirect effects
spat_model=1; % 0 for SAR/SAC, 1 for SDM/GNS model
direct_indirect_effects_estimates(results,WID,spat_model);
%
% Construction of W matrix 1/d^gamma with gamma=3
%
display('Results Inverse Distance to power 3 matrix')
Wgam=WID.^3;
for t=1:T
    t1=(t-1)*N+1;t2=t*N;
    wx(t1:t2,:)=Wgam*x(t1:t2,:);
end
% Normalizes W by largest eigenvalue
lambda=eig(Wgam);
Wgam=Wgam/max(lambda); 
%
% Create WX variables for Wgam matrix
for t=1:T
    t1=(t-1)*N+1;t2=t*N;
    wx(t1:t2,:)=Wgam*x(t1:t2,:);
end
%
% Estimation of SDM, SDEM, and GNS models using inverse distance matrix Wgam
%
% ----------------------------------------------------------------------------------------
% Spatial Durbin model SDM
info.lflag=0; % required for exact results
info.model=3; % sfe + tfe
info.fe=0; % Do not print intercept and fixed effects; use info.fe=1 to turn on
results=sar_panel_FE(y,[x wx],Wgam,T,info); 
% Print out coefficient estimates
vnames=strvcat('logcit','logp','logy','W*logp','W*logy'); % variable names
prt_sp(results,vnames,1);
% Print out direct and indirect effects
spat_model=1; % 0 for SAR/SAC, 1 for SDM/GNS model
direct_indirect_effects_estimates(results,Wgam,spat_model);
% ----------------------------------------------------------------------------------------
% Spatial Durbin error model
info.lflag=0; % required for exact results
info.model=3; % sfe + tfe
info.fe=0; % Do not print intercept and fixed effects; use info.fe=1 to turn on
results=sem_panel_FE(y,[x wx],Wgam,T,info); 
% Print out coefficient estimates
vnames=strvcat('logcit','logp','logy','W*logp','W*logy'); % variable names
prt_sp(results,vnames,1);
% Direct and indirect effects already determined by SDEM
% ----------------------------------------------------------------------------------------
% General nesting spatial model GNS
info.lflag=0; % required for exact results
info.model=3; % sfe + tfe
info.fe=0; % Do not print intercept and fixed effects; use info.fe=1 to turn on
results=sac_panel_FE(y,[x wx],Wgam,T,info);
% Print out coefficient estimates
vnames=strvcat('logcit','logp','logy','W*logp','W*logy'); % variable names
prt_sp(results,vnames,1);
% Print out direct and indirect effects
spat_model=1; % 0 for SAR/SAC, 1 for SDM/GNS model
direct_indirect_effects_estimates(results,Wgam,spat_model);
%
% Bayesian approach to determine best combination of model and W matrix applied to 
% static models
% 
% beta prior
display('Bayesian results per W matrix')
prior.c = 1.01;
prior.d = 1.01;
opt.tol = 1e-3; opt.disp = 0;
incr = 0.001;
in.cnames = strvcat('W-miles log marginals','model probs');
in.rnames = strvcat('Models','sar','sdm','sem','sdem');
ted = 1; % set ted=0 for model with spatial fixed effects without time dummies, % set ted=1 for model with spatial and time period fixed effects
model=3; % sfe + tfe
[yf,xf,meanny,meannx,meanty,meantx]=demean(y,x,N,T,model);

WBC=W; % keep for later
[n,junk] = size(W);
lambda = eigs(sparse(W),speye(n),1,'SR',opt); 
rmin = real(1/lambda) + incr;   
rmax = 1.0 - incr;
[rmin rmax]
out=lndetfull(sparse(W),rmin,rmax);
result = log_marginal_panelprob(yf,xf,W,N,T,model,out,prior,rmin,rmax,incr); 
in.cnames = strvcat('W-miles log marginals','model probs');
in.rnames = strvcat('Models','sar','sdm','sem','sdem');
mprint([result.lmarginal result.probs],in);
lmarginal=[result.lmarginal];

W=WID;
[n,junk] = size(W);
lambda = eigs(sparse(W),speye(n),1,'SR',opt); 
rmin = real(1/lambda) + incr;   
rmax = 1.0 - incr;
[rmin rmax]
out=lndetfull(sparse(W),rmin,rmax);
result = log_marginal_panelprob(yf,xf,W,N,T,model,out,prior,rmin,rmax,incr); 
in.cnames = strvcat('W-miles log marginals','model probs');
in.rnames = strvcat('Models','sar','sdm','sem','sdem');
mprint([result.lmarginal result.probs],in);
lmarginal=[lmarginal;result.lmarginal];

W=Wgam;
[n,junk] = size(W);
lambda = eigs(sparse(W),speye(n),1,'SR',opt); 
rmin = real(1/lambda) + incr;   
rmax = 1.0 - incr;
[rmin rmax]
out=lndetfull(sparse(W),rmin,rmax);
result = log_marginal_panelprob(yf,xf,W,N,T,model,out,prior,rmin,rmax,incr); 
in.cnames = strvcat('W-miles log marginals','model probs');
in.rnames = strvcat('Models','sar','sdm','sem','sdem');
mprint([result.lmarginal result.probs],in);
lmarginal=[lmarginal;result.lmarginal];

nmodels = length(lmarginal);
adj = max(lmarginal(:,1));
madj = matsub(lmarginal,adj);
xx = exp(madj);
% compute posterior probabilities
psum = sum(xx);
probs = [matdiv(xx,psum)];
display('Overall results static models and W; 4 rows (sar,sdm,sem,sdem) for each W')
[lmarginal/1000 probs]

% Switch to dynamic models

[yf,xf,meanny,meannx,meanty,meantx]=demean(y(N+1:end),[y(1:end-N) x(N+1:end,:)],N,T-1,model);

W=WBC;
[n,junk] = size(W);
lambda = eigs(sparse(W),speye(n),1,'SR',opt); 
rmin = real(1/lambda) + incr;   
rmax = 1.0 - incr;
[rmin rmax]
out=lndetfull(sparse(W),rmin,rmax);
result = log_marginal_panelprob(yf,xf,W,N,T-1,model,out,prior,rmin,rmax,incr); 
in.cnames = strvcat('W-miles log marginals','model probs');
in.rnames = strvcat('Models','sar','sdm','sem','sdem');
mprint([result.lmarginal result.probs],in);
lmarginal=[result.lmarginal];

W=WID;
[n,junk] = size(W);
lambda = eigs(sparse(W),speye(n),1,'SR',opt); 
rmin = real(1/lambda) + incr;   
rmax = 1.0 - incr;
[rmin rmax]
out=lndetfull(sparse(W),rmin,rmax);
result = log_marginal_panelprob(yf,xf,W,N,T-1,model,out,prior,rmin,rmax,incr); 
in.cnames = strvcat('W-miles log marginals','model probs');
in.rnames = strvcat('Models','sar','sdm','sem','sdem');
mprint([result.lmarginal result.probs],in);
lmarginal=[lmarginal;result.lmarginal];

W=Wgam;
[n,junk] = size(W);
lambda = eigs(sparse(W),speye(n),1,'SR',opt); 
rmin = real(1/lambda) + incr;   
rmax = 1.0 - incr;
[rmin rmax]
out=lndetfull(sparse(W),rmin,rmax);
result = log_marginal_panelprob(yf,xf,W,N,T-1,model,out,prior,rmin,rmax,incr); 
in.cnames = strvcat('W-miles log marginals','model probs');
in.rnames = strvcat('Models','sar','sdm','sem','sdem');
mprint([result.lmarginal result.probs],in);
lmarginal=[lmarginal;result.lmarginal];

nmodels = length(lmarginal);
adj = max(lmarginal(:,1));
madj = matsub(lmarginal,adj);
xx = exp(madj);
% compute posterior probabilities
psum = sum(xx);
probs = [matdiv(xx,psum)];
display('Overall results dynamic models and W; 4 rows (sar,sdm,sem,sdem) for each W')
[lmarginal/1000 probs]

%
% END PART II: Bayesian comparison tests, with W=BC, ID, ID^3
%
% Remove stop in line below after you answered questions 5-6.
%Stop=secondpart

%
% PART III: Estimation of dynamic spatial Durbin models with sfe and tfe, without common factors
%
%%%
fprintf(1,'DSPD with sfe and tfe \n');
W=WID; % Inverse distance matrix is used in the estimations below
%%%
display('CD test raw data');
y_ni      = reshape(y,N,T)'                  ;
R         = corrcoef(y_ni) - diag(ones(N,1)) ;
rho_h_ij  = sum(sum(triu(R))')               ;
rho_h_bar = (2/(N*(N-1)))*rho_h_ij           ;
CD_stat   = sqrt((T*N*(N-1))/2)*rho_h_bar    ;
disp('Cross-Sectional Dependence (residuals)')
disp(['rho_hat_bar         ',num2str(rho_h_bar,'% 10.3f')])
disp(['CD statistic        ',num2str(CD_stat,'% 10.3f')])
%---
for t=1:T
    t1=(t-1)*N+1;t2=t*N;
    Wy(t1:t2,1)=W*y(t1:t2,1);
end
info.lflag=0;
info.tl=1;
info.stl=1;
info.ted=1;
info.dyn=1;
info.model=3;
info.fe=0;
info.bc=0;
results=sar_panel_FE(y(N+1:end),[y(1:end-N) Wy(1:end-N) x(N+1:end,:) wx(N+1:end,:)],W,T-1,info);
vnames=strvcat('logcit','timelag logcit','spacetimelag logcit','logp','logy','W*logp','W*logy');
results1=sar_jihai_time(y(1:nobs),[x(N+1:nobs,:) wx(N+1:nobs,:)],W,info);
results.beta=results1.theta1(1:end-2);
results.rho=results1.theta1(end-1);
results.tstat=results1.tstat1(1:end-1);
prt_sp(results,vnames,1);
btemp=results1.theta1;
varcov=results1.varcov;
%
% Direct and indirect effects estimates
% st=short term, lt=long term
% c=(long term) convergence effect of dependent variable
%
% Direct and indirect effects estimates
% st=short term, lt=long term
% c=(long term) convergence effect of dependent variable
%
NSIM=1000;
px=K;
[npar dummy]=size(btemp);
simresults=zeros(npar-1,NSIM);
simdirst=zeros(px,NSIM);
simindst=zeros(px,NSIM);
simtotst=zeros(px,NSIM);
simdirlt=zeros(px,NSIM);
simindlt=zeros(px,NSIM);
simtotlt=zeros(px,NSIM);
simdirc=zeros(1,NSIM);
simindc=zeros(1,NSIM);
simtotc=zeros(1,NSIM);
for sim=1:NSIM
    parms = chol(varcov)'*randn(size(btemp)) + btemp;
    deltasim = parms(npar-1,1); % coef WY(t)
    betasim = parms(3:npar-2,1);
    tausim = parms(1,1); % Coef Y(t-1)
    etasim = parms(2,1); % Coef WY(t-1)
    simresults(:,sim)=[tausim;etasim;betasim;deltasim];
    SS=(eye(N)-deltasim*W)\eye(N);
    SC=SS*((tausim-1)*eye(N)+(deltasim+etasim)*W);
    simdirc(1,sim)=sum(diag(SC))/N; % average direct effect
    simindc(1,sim)=sum(sum(SC,2)-diag(SC))/N; % average indirect effect
    simtotc(1,sim)=simdirc(1,sim)+simindc(1,sim);
    for p=1:px
        C=zeros(N,N);
        for i=1:N
            for j=1:N
                if (i==j) C(i,j)=betasim(p);
                else C(i,j)=betasim(p+2)*W(i,j);
                end
            end
        end
        SC=SS*C;
        simdirst(p,sim)=sum(diag(SC))/N; % average direct effect
        simindst(p,sim)=sum(sum(SC,2)-diag(SC))/N; % average indirect effect
        simtotst(p,sim)=simdirst(p,sim)+simindst(p,sim);
        SC=((1-tausim)*eye(N)-(deltasim+etasim)*W)\C;
        simdirlt(p,sim)=sum(diag(SC))/N; % average direct effect
        simindlt(p,sim)=sum(sum(SC,2)-diag(SC))/N; % average indirect effect
        simtotlt(p,sim)=simdirlt(p,sim)+simindlt(p,sim);        
    end
end
fprintf(1,'   Dir.eff   t-value   Ind.eff   t-value   Tot.eff   t-value \n');
fprintf(1,'Convergence effect \n');
[mean(simdirc,2) mean(simdirc,2)./std(simdirc,0,2) mean(simindc,2) mean(simindc,2)./std(simindc,0,2)...
    mean(simtotc,2) mean(simtotc,2)./std(simtotc,0,2)]
fprintf(1,'Short term effects \n');
[mean(simdirst,2) mean(simdirst,2)./std(simdirst,0,2) mean(simindst,2) mean(simindst,2)./std(simindst,0,2)...
    mean(simtotst,2) mean(simtotst,2)./std(simtotst,0,2)]
fprintf(1,'Long term effects \n');
[mean(simdirlt,2) mean(simdirlt,2)./std(simdirlt,0,2) mean(simindlt,2) mean(simindlt,2)./std(simindlt,0,2)...
    mean(simtotlt,2) mean(simtotlt,2)./std(simtotlt,0,2)]
clear simresults simdirc simindc simtotc simdirst simindst simtotst simdirlt simindlt simtotlt;
res= results1.respaul1;
% CD test
display('CD test residuals');
n=N-1;
y_ni      = reshape(res,n,T-1)'              ;
R         = corrcoef(y_ni) - diag(ones(n,1)) ;
rho_h_ij  = sum(sum(triu(R))')               ;
rho_h_bar = (2/(n*(n-1)))*rho_h_ij           ;
CD_stat   = sqrt(((T-1)*n*(n-1))/2)*rho_h_bar;
disp('Cross-Sectional Dependence (residuals)')
disp(['rho_hat_bar         ',num2str(rho_h_bar,'% 10.3f')])
disp(['CD statistic        ',num2str(CD_stat,'% 10.3f')])
fprintf(1,'End results DSPD with sfe and tfe \n');
%
% END PART III: Estimation of dynamic spatial Durbin models with sfe and tfe, without common factors
%
% Remove stop in line below after you answered question 7.
%Stop=thirdpart
%
% PART IV: Include common factors
%
%
% CF in the form of CSAs
%
fprintf(1,' \n');
fprintf(1,'DSPD with CSAs \n');
ym=zeros(T,1);
xm=zeros(T,K);
yx=zeros(nobs,N);
x1=zeros(nobs,N);x2=zeros(nobs,N);
for t=1:T
    t1=(t-1)*N+1;t2=t*N;
    ym(t)=mean(y(t1:t2));
    xm(t,:)=mean(x(t1:t2,:));
end
ysample=kron(ym,ones(N,1));
xsample=kron(xm,ones(N,1));
for t=1:T
    t1=(t-1)*N+1;t2=t*N;
    for i=1:N
        yx(t1-1+i,i)=ysample(t1-1+i,1);
        x1(t1-1+i,i)=xsample(t1-1+i,1); %CF=CSA of first x-var, not used below
        x2(t1-1+i,i)=xsample(t1-1+i,2); %CF=CSA of second x-var, not used below
    end
end
%
% Estimation of dynamic spatial Durbin models with sfe and tfe, with common factors as CSA of y at time t and t-1.
%
init = ['y(t-1)      ';'Wy(t-1)     ';
        'c-y(t)1     ';'c-y(t)2     ';'c-y(t)3     ';'c-y(t)4     ';'c-y(t)5     ';'c-y(t)6     ';'c-y(t)7     ';'c-y(t)8     ';'c-y(t)9     ';'c-y(t)10    ';
        'c-y(t)11    ';'c-y(t)12    ';'c-y(t)13    ';'c-y(t)14    ';'c-y(t)15    ';'c-y(t)16    ';'c-y(t)17    ';'c-y(t)18    ';'c-y(t)19    ';'c-y(t)20    ';
        'c-y(t)21    ';'c-y(t)22    ';'c-y(t)23    ';'c-y(t)24    ';'c-y(t)25    ';'c-y(t)26    ';'c-y(t)27    ';'c-y(t)28    ';'c-y(t)29    ';'c-y(t)30    ';
        'c-y(t)31    ';'c-y(t)32    ';'c-y(t)33    ';'c-y(t)34    ';'c-y(t)35    ';'c-y(t)36    ';'c-y(t)37    ';'c-y(t)38    ';'c-y(t)39    ';'c-y(t)40    ';
        'c-y(t)41    ';'c-y(t)42    ';'c-y(t)43    ';'c-y(t)44    ';'c-y(t)45    ';'c-y(t)46    ';'c-y(t-1)1   ';'c-y(t-1)2   ';'c-y(t-1)3   ';'c-y(t-1)4   ';
        'c-y(t-1)5   ';'c-y(t-1)6   ';'c-y(t-1)7   ';'c-y(t-1)8   ';'c-y(t-1)9   ';'c-y(t-1)10  ';'c-y(t-1)11  ';'c-y(t-1)12  ';'c-y(t-1)13  ';'c-y(t-1)14  ';
        'c-y(t-1)15  ';'c-y(t-1)16  ';'c-y(t-1)17  ';'c-y(t-1)18  ';'c-y(t-1)19  ';'c-y(t-1)20  ';'c-y(t-1)21  ';'c-y(t-1)22  ';'c-y(t-1)23  ';'c-y(t-1)24  ';
        'c-y(t-1)25  ';'c-y(t-1)26  ';'c-y(t-1)27  ';'c-y(t-1)28  ';'c-y(t-1)29  ';'c-y(t-1)30  ';'c-y(t-1)31  ';'c-y(t-1)32  ';'c-y(t-1)33  ';'c-y(t-1)34  ';
        'c-y(t-1)35  ';'c-y(t-1)36  ';'c-y(t-1)37  ';'c-y(t-1)38  ';'c-y(t-1)39  ';'c-y(t-1)40  ';'c-y(t-1)41  ';'c-y(t-1)42  ';'c-y(t-1)43  ';'c-y(t-1)44  ';
        'c-y(t-1)45  ';'c-y(t-1)46  ';'logp        ';'logI        ';'W*logp      ';'W*logI      ';'W*y(t)      '];
results=sar_jihai(y(1:nobs),[yx(N+1:end,1:N) yx(1:end-N,1:N) x(N+1:end,:) wx(N+1:end,:)],W,info);
resultats = [results.theta results.theta1 results.tstat1];
[nr,~]    = size(resultats);
resul     = resultats(1:nr,:);
disp('Yu, de Jong and Lee (2008) ')
disp('(B-C means Bias-Corrected) ')
disp(' ')
disp('            Not B-C      B-C    t-stats')
disp([init,num2str(resul(1:nr-1,:),'% 10.4f')])
disp(' ')
initstat=['R-squared   ';'R2 corrected';'LogL uncor  ';'LogLcorr    ';'sigma2      ';'sigma2 BC   '];
statistics=[results.rsqr results.corr2 results.lik results.lik1 results.theta(end) results.theta1(end)]'; 
disp([initstat,num2str(statistics,'% 15.4f')])
res= results.respaul1;
disp(' ')
n=N;
y_ni      = reshape(res,n,T-1)'              ;
R         = corrcoef(y_ni) - diag(ones(n,1)) ;
rho_h_ij  = sum(sum(triu(R))')               ;
rho_h_bar = (2/(n*(n-1)))*rho_h_ij         ;
CD_stat   = sqrt(((T-1)*n*(n-1))/2)*rho_h_bar;
disp('Cross-Sectional Dependence (residuals)')
disp(['rho_hat_bar         ',num2str(rho_h_bar,'% 10.3f')])
disp(['CD statistic        ',num2str(CD_stat,'% 10.3f')])

btemp=results.theta1;
varcov=results.varcov;
tau = results.theta1(1,1);
eta = results.theta1(2,1);
rho = results.theta1(end,1);
varcov=results.varcov;
NSIM=1000;
px=K;
[npar dummy]=size(btemp);
simresults=zeros(npar,NSIM);
simdirst=zeros(px,NSIM);
simindst=zeros(px,NSIM);
simtotst=zeros(px,NSIM);
simdirlt=zeros(px,NSIM);
simindlt=zeros(px,NSIM);
simtotlt=zeros(px,NSIM);
simdirc=zeros(1,NSIM);
simindc=zeros(1,NSIM);
simtotc=zeros(1,NSIM);
for sim=1:NSIM
    parms = chol(varcov)'*randn(size(btemp)) + btemp;
    deltasim = parms(end,1); % coef WY(t)
    betasim = parms(3:end-1,1);
    tausim = parms(1,1); % Coef Y(t-1)
    etasim = parms(2,1); % Coef WY(t-1)
    simresults(:,sim)=[tausim;etasim;betasim;deltasim];
    SS=(eye(N)-deltasim*W)\eye(N);
    SC=SS*((tausim-1)*eye(N)+(deltasim+etasim)*W);
    simdirc(1,sim)=sum(diag(SC))/N; % average direct effect
    simindc(1,sim)=sum(sum(SC,2)-diag(SC))/N; % average indirect effect
    simtotc(1,sim)=simdirc(1,sim)+simindc(1,sim);
    for p=1:px
        C=zeros(N,N);
        for i=1:N
            for j=1:N
                if (i==j) C(i,j)=betasim(p);
                else C(i,j)=betasim(p+px)*W(i,j);
                end
            end
        end
        SC=SS*C;
        simdirst(p,sim)=sum(diag(SC))/N; % average direct effect
        simindst(p,sim)=sum(sum(SC,2)-diag(SC))/N; % average indirect effect
        simtotst(p,sim)=simdirst(p,sim)+simindst(p,sim);
        SC=((1-tausim)*eye(N)-(deltasim+etasim)*W)\C;
        simdirlt(p,sim)=sum(diag(SC))/N; % average direct effect
        simindlt(p,sim)=sum(sum(SC,2)-diag(SC))/N; % average indirect effect
        simtotlt(p,sim)=simdirlt(p,sim)+simindlt(p,sim);        
    end
end
fprintf(1,'   Dir.eff   t-value   Ind.eff   t-value   Tot.eff   t-value \n');
fprintf(1,'Convergence effect \n');
[mean(simdirc,2) mean(simdirc,2)./std(simdirc,0,2) mean(simindc,2) mean(simindc,2)./std(simindc,0,2)...
    mean(simtotc,2) mean(simtotc,2)./std(simtotc,0,2)]
fprintf(1,'Short term effects \n');
[mean(simdirst,2) mean(simdirst,2)./std(simdirst,0,2) mean(simindst,2) mean(simindst,2)./std(simindst,0,2)...
    mean(simtotst,2) mean(simtotst,2)./std(simtotst,0,2)]
fprintf(1,'Long term effects \n');
[mean(simdirlt,2) mean(simdirlt,2)./std(simdirlt,0,2) mean(simindlt,2) mean(simindlt,2)./std(simindlt,0,2)...
    mean(simtotlt,2) mean(simtotlt,2)./std(simtotlt,0,2)]
fprintf(1,'End results DSPD with CSAs \n');
%
% Comon factors (in this case R=2) as principal components
%
fprintf(1,' \n');
fprintf(1,'DSPD with PCs \n');
init = ['y(t-1)      ';'Wy(t-1)     ';
        'logp        ';'logy        ';'W*logp      ';'W*logy      ';
        'Wy(t)       ';'W*error     '];
vnames=strvcat('logcit','timelag logcit','spacetimelag logcit','logp','logy','W*logp','W*logy');
lambda0 = 0.1;
gamma0  = 0.1;
rho0    = 0.6;
alpha0  = 0.1;
beta0   = zeros(2*K,1); %change3
start = [gamma0;rho0;beta0;lambda0;alpha0];
TolX=0.0001;
Y=reshape(y(N+1:end),N,T-1);
X = zeros(2+2*K,N,T-1); %change4
X(1,:,:) = reshape(y(1:end-N),N,T-1);
X(2,:,:) = reshape(Wy(1:end-N),N,T-1);
for i=1:K %change5
    X(i+2,:,:) = reshape(x(N+1:end,i),N,T-1);
    X(i+2+K,:,:) = reshape(wx(N+1:end,i),N,T-1);
end
R=2; % # number of CF
[beta,bcorr,Vbeta,gamma,f,exitflag,stats,resid] = SFactors(Y,W,W,X,R,start,TolX);
exitflag
resul=[beta-bcorr sqrt(diag(Vbeta)) (beta-bcorr)./sqrt(diag(Vbeta))]; % Note t-value excluding bcorrection
disp('             Coef      Sdev      t-stat')
disp([init,num2str(resul,'% 10.4f')])
disp('R2')
stats(1,1)
disp('LogL')
stats(3,1)
y_ni      = reshape(resid,N,T-1)'              ;
R         = corrcoef(y_ni) - diag(ones(N,1)) ;
rho_h_ij  = sum(sum(triu(R))')               ;
rho_h_bar = (2/(N*(N-1)))*rho_h_ij           ;
CD_stat   = sqrt(((T-1)*N*(N-1))/2)*rho_h_bar;
disp('Cross-Sectional Dependence (residuals)')
disp(['rho_hat_bar         ',num2str(rho_h_bar,'% 10.3f')])
disp(['CD statistic        ',num2str(CD_stat,'% 10.3f')])
%
% Direct and indirect effects estimates
% st=short term, lt=long term
% c=(long term) convergence effect of dependent variable
%
% Direct and indirect effects estimates
% st=short term, lt=long term
% c=(long term) convergence effect of dependent variable
%
NSIM=1000;
px=K;
btemp=beta-bcorr;
[npar dummy]=size(btemp);
simresults=zeros(npar-1,NSIM);
simdirst=zeros(px,NSIM);
simindst=zeros(px,NSIM);
simtotst=zeros(px,NSIM);
simdirlt=zeros(px,NSIM);
simindlt=zeros(px,NSIM);
simtotlt=zeros(px,NSIM);
simdirc=zeros(1,NSIM);
simindc=zeros(1,NSIM);
simtotc=zeros(1,NSIM);
for sim=1:NSIM
    parms = chol(Vbeta)'*randn(size(btemp)) + btemp;
    deltasim = parms(npar-1,1); % coef WY(t)
    betasim = parms(3:npar-2,1);
    tausim = parms(1,1); % Coef Y(t-1)
    etasim = parms(2,1); % Coef WY(t-1)
    simresults(:,sim)=[tausim;etasim;betasim;deltasim];
    SS=(eye(N)-deltasim*W)\eye(N);
    SC=SS*((tausim-1)*eye(N)+(deltasim+etasim)*W);
    simdirc(1,sim)=sum(diag(SC))/N; % average direct effect
    simindc(1,sim)=sum(sum(SC,2)-diag(SC))/N; % average indirect effect
    simtotc(1,sim)=simdirc(1,sim)+simindc(1,sim);
    for p=1:px
        C=zeros(N,N);
        for i=1:N
            for j=1:N
                if (i==j) C(i,j)=betasim(p);
                else C(i,j)=betasim(p+px)*W(i,j);
                end
            end
        end
        SC=SS*C;
        simdirst(p,sim)=sum(diag(SC))/N; % average direct effect
        simindst(p,sim)=sum(sum(SC,2)-diag(SC))/N; % average indirect effect
        simtotst(p,sim)=simdirst(p,sim)+simindst(p,sim);
        SC=((1-tausim)*eye(N)-(deltasim+etasim)*W)\C;
        simdirlt(p,sim)=sum(diag(SC))/N; % average direct effect
        simindlt(p,sim)=sum(sum(SC,2)-diag(SC))/N; % average indirect effect
        simtotlt(p,sim)=simdirlt(p,sim)+simindlt(p,sim);        
    end
end
fprintf(1,'   Dir.eff   t-value   Ind.eff   t-value   Tot.eff   t-value \n');
fprintf(1,'Convergence effect \n');
[mean(simdirc,2) mean(simdirc,2)./std(simdirc,0,2) mean(simindc,2) mean(simindc,2)./std(simindc,0,2)...
    mean(simtotc,2) mean(simtotc,2)./std(simtotc,0,2)]
fprintf(1,'Short term effects \n');
[mean(simdirst,2) mean(simdirst,2)./std(simdirst,0,2) mean(simindst,2) mean(simindst,2)./std(simindst,0,2)...
    mean(simtotst,2) mean(simtotst,2)./std(simtotst,0,2)]
fprintf(1,'Long term effects \n');
[mean(simdirlt,2) mean(simdirlt,2)./std(simdirlt,0,2) mean(simindlt,2) mean(simindlt,2)./std(simindlt,0,2)...
    mean(simtotlt,2) mean(simtotlt,2)./std(simtotlt,0,2)]
clear simresults simdirc simindc simtotc simdirst simindst simtotst simdirlt simindlt simtotlt;
fprintf(1,'End results DSPD with PCs \n');